
#include "main.h"
#include "chaos.h"

///::title:: Wave equation with stochastic coefficient (1D version)
///::vars:: NX=512 MAXT=2.0 KL=1
///::consts:: MAXX=2 N_UNIFORM=KL N_NORMAL=0
///::bc:: REPEAT

// u_0     = sin(pi*x);
// (u_0)_t = 0.0;

void init_cell(PrimitiveVars &v, real x, real y, real z)
{
  v.v[0] = pi*cos(pi*x);
  v.w    = 0.0;
  
  // Karhunen–Loeve expantion for coefficient k consisting of KL basis functions
  v.k    = 2.0;
  for (int m=1; m<=KL; m++)
    v.k += 0.1 * pow(m,-1.1) * sin(m*pi*x) * (2*chaos.uniform[m-1] - 1);
  
  v.rho  = 1.0;
}
