
#include "equation.h"
#include "hll_util.h"
#include "solver.h"

///::title:: HLL3L solver (Linde 3 wave) | for MHD
///::depends:: hll_util-mhd
///::consts:: HLL_dt=1

void solver_calc_flux(ConservedVars &F, vector &us, vector &Bs,
		      const AllVariables &lV, const AllVariables &rV,
		      int i, int j, int k, int d)
{
  real sl, sr, cs;
  ConservedVars lF, rF;
  
  hll_speeds(lV, rV, d, sl, sr, cs);
  calc_point_flux(lF, lV, d);
  calc_point_flux(rF, rV, d);
  ConservedVars Us = (sr*rV.U - sl*lV.U - rF + lF)/(sr - sl);
  us = Us.m / Us.rho;
  Bs = Us.B;
  if (sl > 0) F = lF;
  else if (sr < 0) F = rF;
  else {
    real sm = us[d];
    ConservedVars dU = rV.U - lV.U;
    ConservedVars dF = rF - lF;
    real s = (dF - sm*dU).norm() / (dU.norm() + 0.000000001);
    real alpha = max(0, 1-s/(cs + 0.000000001));
    real f;
    if (sm > 0) {
      f = sl*(alpha*sm + (1-alpha)*sr);
      //      Us -= (sr-sm)*dU/(sr-sl);
    } else {
      f = sr*(alpha*sm + (1-alpha)*sl);
      //      Us += (sm-sl)*dU/(sr-sl);
    }
    F = (sr*lF - sl*rF + f*dU)/(sr-sl);
  }
}
