
#include "topography.h"

#include "equation.h"
#include "multi.h"
#include "chaos.h"

#include "topography-wavelet.h"

///::title:: Haar wavelet representation (2D version)
///::vars:: WL=7
///::consts:: WL_DECAY=1.3 WL_MIN_CORR_LEVEL=3
///::consts:: T_UNIFORM=(1<<(2*WL)) T_NORMAL=0 INTERPOLATE_TOPOGRAPHY=1
///::depends:: topography-wavelet
///::macro:: LEVEL_INIT_HOOK topography-wavelet.h wavelet_init(NXl*MULTIXl, NYl*MULTIYl);

real b (real x, real y) {
  
  real value = 0.0;
  long_int counter = 0;
  
  value += 0.1 * (2*chaos.t_uniform[0] - 1);
  counter++;
  
  for (int m=max(1,WL_MIN_CORR_LEVEL); m<=WL; m++) {
    
    // isotropic part
    for (int jx=0; jx<J[m-1]; jx++)
      for (int jy=0; jy<J[m-1]; jy++)
        value += C[m] * wavelet(x/MAXX,m,jx) * wavelet(y/MAXY,m,jy) * (2*chaos.t_uniform[counter++] - 1);
    
    // upper unisotropic part
    for (int jx=0; jx<J[m-1]; jx++)
      for (int jc=0; jc<J[m-1]; jc++)
        value += C[m] * wavelet(x/MAXX,m,jx) * char_function(y/MAXY,m-1,jc) * (2*chaos.t_uniform[counter++] - 1);
    
    // lower unisotropic part
    for (int jy=0; jy<J[m-1]; jy++)
      for (int jc=0; jc<J[m-1]; jc++)
        value += C[m] * wavelet(y/MAXX,m,jy) * char_function(x/MAXX,m-1,jc) * (2*chaos.t_uniform[counter++] - 1);
  }
  
  return BED_OFFSET + BED_SCALING * 0.1 * value;
}
